import json
import os

__VERSION__ = (2018, 7, 23)
CURRENT_DIR = os.path.dirname(os.path.realpath(__file__))
STOP_WORDS_DIR = os.path.join(CURRENT_DIR, 'stop-words')
STOP_WORDS_CACHE = {}

with open(os.path.join(STOP_WORDS_DIR, 'languages.json'), 'rb') as map_file:
    buffer = map_file.read()
    buffer = buffer.decode('ascii')
    LANGUAGE_MAPPING = json.loads(buffer)

AVAILABLE_LANGUAGES = list(LANGUAGE_MAPPING.values())


def get_version():
    """
    :rtype: basestring
    """
    return ".".join(str(v) for v in __VERSION__)


class StopWordError(Exception):
    pass


def get_stop_words(language, cache=True):
    """
    :type language: basestring

    :rtype: list
    """
    try:
        language = LANGUAGE_MAPPING[language]
    except KeyError:
        if language not in AVAILABLE_LANGUAGES:
            raise StopWordError('{0}" language is unavailable.'.format(
                language
            ))

    if cache and language in STOP_WORDS_CACHE:
        return STOP_WORDS_CACHE[language]

    language_filename = os.path.join(STOP_WORDS_DIR, language + '.txt')
    try:
        with open(language_filename, 'rb') as language_file:
            stop_words = [line.decode('utf-8').strip()
                          for line in language_file.readlines()]
            stop_words = apply_filters(stop_words, language)
    except IOError:
        raise StopWordError(
            '{0}" file is unreadable, check your installation.'.format(
                language_filename
            )
        )

    if cache:
        STOP_WORDS_CACHE[language] = stop_words

    return stop_words

_filters = {None: []}


def apply_filters(stopwords, language):
    """
    Apply registered filters to stopwords
    :param stopwords: list
    :param language: string
    :return: filtered stopwords
    """
    if language in _filters:
        for func in _filters[language]:
            stopwords = func(stopwords)

    for func in _filters[None]:
        stopwords = func(stopwords, language)

    return stopwords


def add_filter(func, language=None):
    """
    Register filters for specific language.
    If language == None the filter applies for all languages.
    Filter will not apply for stop words in cache.
    :param func: callable
    :param language: string|None
    :return:
    """
    if language not in _filters:
        _filters[language] = []
    _filters[language].append(func)


def remove_filter(func, language=None):
    """
    :param func:
    :param language:
    :return:
    """
    if not (language in _filters and func in _filters[language]):
        return False
    _filters[language].remove(func)
    return True


def safe_get_stop_words(language):
    """
    :type language: basestring

    :rtype: list
    """
    try:
        return get_stop_words(language)
    except StopWordError:
        return []



"""
Tests for stop-words
"""
import random
from unittest import TestCase
from unittest import TestSuite
from unittest import TestLoader

import stop_words
from stop_words import get_stop_words
from stop_words import safe_get_stop_words
from stop_words import StopWordError
from stop_words import LANGUAGE_MAPPING
from stop_words import AVAILABLE_LANGUAGES


class StopWordsTestCase(TestCase):
    number_of_english_stop_words = 174

    def test_get_stop_words(self):
        sw = get_stop_words('english')
        self.assertEqual(len(sw), self.number_of_english_stop_words)

    def test_get_stop_words_language_mapping(self):
        sw = get_stop_words('en')
        self.assertEqual(len(sw), self.number_of_english_stop_words)
        self.assertEqual(sw, get_stop_words('english'))

    def test_get_stop_words_cache(self):
        self.assertFalse('french' in stop_words.STOP_WORDS_CACHE)
        sw = get_stop_words('fr')
        self.assertTrue('french' in stop_words.STOP_WORDS_CACHE)
        original_stop_words_dir = stop_words.STOP_WORDS_DIR
        stop_words.STOP_WORDS_DIR = 'not-existing-directory'
        self.assertEqual(sw, get_stop_words('french'))
        stop_words.STOP_WORDS_DIR = original_stop_words_dir
        try:
            get_stop_words('klingon')
        except:
            pass
        self.assertFalse('klingon' in stop_words.STOP_WORDS_CACHE)

    def test_get_stop_words_unavailable_language(self):
        self.assertRaises(StopWordError, get_stop_words, 'sindarin')

    def test_get_stop_words_install_issue(self):
        original_stop_words_dir = stop_words.STOP_WORDS_DIR
        stop_words.STOP_WORDS_DIR = 'not-existing-directory'
        self.assertRaises(StopWordError, get_stop_words, 'german')
        stop_words.STOP_WORDS_DIR = original_stop_words_dir

    def test_safe_get_stop_words(self):
        self.assertRaises(StopWordError, get_stop_words, 'huttese')
        self.assertEqual(safe_get_stop_words('huttese'), [])

    def test_random_language_stop_words_load(self):
        languages = list(LANGUAGE_MAPPING.keys()) + list(AVAILABLE_LANGUAGES)
        sample = random.sample(languages, len(languages))
        for language in sample:
            stop_words = safe_get_stop_words(language)
            self.assertTrue(
                len(stop_words) > 0,
                'Cannot load stopwords for {0} language'.format(language)
            )

    def test_filters(self):
            language = 'en'
            before = get_stop_words(language, False)
            letter = random.choice(random.choice(before))

            def remove_letter(stopwords, language):
                return [word for word in stopwords if letter not in word]
            stop_words.add_filter(remove_letter)
            after = get_stop_words(language, False)
            for stopword in after:
                self.assertFalse(letter in stopword)
            self.assertTrue(stop_words.remove_filter(remove_letter))


loader = TestLoader()

test_suite = TestSuite(
    [
        loader.loadTestsFromTestCase(StopWordsTestCase),
    ]
)



from setuptools import setup, find_packages

setup(
    name='stop-words',
    version=__import__("stop_words").get_version(),
    description='Get list of common stop words in various languages in Python',
    long_description=open('README.rst').read(),
    license=open('LICENSE').read(),
    author='Alireza Savand',
    author_email='alireza.savand@gmail.com',
    url='https://github.com/Alir3z4/python-stop-words',
    packages=find_packages(),
    zip_safe=False,
    package_data={
        'stop_words': [
            'stop-words/*.txt',
            'stop-words/languages.json',
        ]
    },
    classifiers=[
        'Programming Language :: Python',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Topic :: Software Development',
        'Development Status :: 6 - Mature',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Text Processing',
        'Topic :: Text Processing :: Filters',
        'License :: OSI Approved :: BSD License',
    ],
)
